﻿using System;
using System.IO;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization;
using System.Windows.Forms.DataVisualization.Charting;

namespace DigitalMultimeter
{
    public partial class Form1 : Form
    {
        int[] baud_rate_select_datas = new int[12] { 1200, 2400, 3600, 9600, 19200, 38400, 57600, 76800, 115200, 230400, 460800, 921600 };
        const int baud_rate_default_select_idx = 8;
        const int SERIAL_RECEIVED_BUFF_SIZE = 0x1000;
        byte[] serial_received_buff = new byte[SERIAL_RECEIVED_BUFF_SIZE];
        int serial_rcv_data_read_idx = 0;
        int serial_rcv_data_write_idx = 0;
        bool serial_rcv_data_full = false;

        const double GRAPH_TIME_SET_VAL_MIN = 0;
        const double GRAPH_TIME_SET_VAL_MAX = 100;

        // Log file
        bool logfile_out_flag = false;
        StreamWriter sw_serial_log = null;
        const string LOG_FILE_NAME = "\\DMM_LOG_";
        const string LOG_FILE_EXTEN = ".csv";
        string[] LOG_FILE_HEADER = new string[10] { "no", "date", "time", "voltage[V]", "current[A]", "power[W]", "resistance_h[kOhm]", "resistance_l[Ohm]", "capacitance[pF]", "frequency[Hz]" };
        DateTime dt_log_next_output = DateTime.Now;
        int log_output_data_counter = 0;

        public SettingVal mySettingVal = new SettingVal();
        //public SettingVal mySettingValtmp = new SettingVal();
        public CorrectionVal myCorrectionVal = new CorrectionVal();
        //public CorrectionVal myCorrectionValtmp = new CorrectionVal();
        public bool setting_val_input_disabled = false;

        public MeasureVal myMeasureVal = new MeasureVal();
        public bool measure_start_flag = false;
        DateTime dt_MeasureVal_next_update = DateTime.Now;
        const int GRAPH_ITEM_NUM = 7;
        const int GRAPH_ITEM_VOLTAGE_INDEX = 0;
        const int GRAPH_ITEM_CURRENT_INDEX = 1;
        const int GRAPH_ITEM_POWER_INDEX = 2;
        const int GRAPH_ITEM_RESISTANCE_H_INDEX = 3;
        const int GRAPH_ITEM_RESISTANCE_L_INDEX = 4;
        const int GRAPH_ITEM_CAPACITANCE_INDEX = 5;
        const int GRAPH_ITEM_FREQUENCY_INDEX = 6;
        string[] GRAPH_SELECT_ITEM = new string[GRAPH_ITEM_NUM] { "電圧", "電流", "電力", "高抵抗", "低抵抗", "容量", "周波数" };
        string[] GRAPH_Y_UNIT = new string[GRAPH_ITEM_NUM] { "電圧[V]", "電流[A]", "電力[W]", "高抵抗[kOhm]", "低抵抗[Ohm]", "容量[pF]", "周波数[Hz]" };
        double[] GRAPH_Y_MIN = new double[GRAPH_ITEM_NUM] { -20, -2, -40, 0, 0, 0, 0 };
        double[] GRAPH_Y_MAX = new double[GRAPH_ITEM_NUM] { 20, 2, 40, 1000, 10000, 1000000, 20000000 };
        double[] GRAPH_Y_INT = new double[GRAPH_ITEM_NUM] { 5, 0.5, 10, 100, 1000, 100000, 2000000 };
        int graph_select_index = 0;
        const int GRAPH_DATA_BUFF_SIZE = 100;
        double[] graph_data = new double[GRAPH_DATA_BUFF_SIZE];
        int graph_data_read_start_pos = 0;
        int graph_data_next_write_pos = 0;
        int graph_data_num = GRAPH_DATA_BUFF_SIZE;
        bool graph_update_flag = false;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
#if DEBUG
#else
                this.Size = new System.Drawing.Size(1115, 510);

                groupBox1.Visible = false;
#endif

            for (int fi = 0; fi < 64; fi++)
            {
                cmbbx_port.Items.Add(string.Format("COM{0}", fi + 1));
            }
            cmbbx_port.SelectedIndex = 0;
            for (int fi = 0; fi < baud_rate_select_datas.Length; fi++)
            {
                cmbbx_baudrate.Items.Add(baud_rate_select_datas[fi].ToString());
            }
            cmbbx_baudrate.SelectedIndex = baud_rate_default_select_idx;
            for (int fi = 0; fi < GRAPH_SELECT_ITEM.Length; fi++)
            {
                cmbbx_graph_item.Items.Add(GRAPH_SELECT_ITEM[fi]);
            }
            cmbbx_graph_item.SelectedIndex = 0;

            // 初期設定ファイル読み込み
            string file_path = Path.Combine(System.Environment.CurrentDirectory, Constants.INIT_FILE_NAME);
            int func_ret = my_Init_File_Read(file_path);
            if (func_ret == 0)
            {
            }
            else
            {
                mySettingVal.Init();
            }
            // 設定値表示
            my_setting_val_display_set();

            // チャート設定
            // 1.Seriesの追加
            chart1.Series.Clear();
            chart1.Series.Add("level");
            chart1.Series[0].Color = Color.Orange;
            chart1.Series[0].IsVisibleInLegend = false;
            chart1.Series[0].IsValueShownAsLabel = false;
            // 2.グラフのタイプの設定
            chart1.Series["level"].ChartType = System.Windows.Forms.DataVisualization.Charting.SeriesChartType.Line;
            //軸ラベルの設定
            chart1.ChartAreas[0].AxisX.Title = "Time";
            chart1.ChartAreas[0].AxisY.Title = GRAPH_Y_UNIT[graph_select_index];

            //X軸最小値、最大値、目盛間隔の設定
            chart1.ChartAreas[0].AxisX.Minimum = GRAPH_TIME_SET_VAL_MIN;
            chart1.ChartAreas[0].AxisX.Maximum = GRAPH_TIME_SET_VAL_MAX;
            chart1.ChartAreas[0].AxisX.Interval = 20;
            chart1.ChartAreas[0].AxisX.LabelAutoFitMaxFontSize = 10;
            chart1.ChartAreas[0].AxisX.LabelStyle.ForeColor = Color.Black;
            chart1.ChartAreas[0].AxisX.MajorGrid.Enabled = true;
            chart1.ChartAreas[0].AxisX.MajorGrid.LineColor = Color.DarkGray;
            chart1.ChartAreas[0].AxisX.MinorGrid.Enabled = true;
            chart1.ChartAreas[0].AxisX.MinorGrid.LineColor = Color.LightGray;
            //chart1.ChartAreas[0].AxisX.Interval = 1000000;
            //Y軸最小値、最大値、目盛間隔の設定
            chart1.ChartAreas[0].AxisY.LabelStyle.Enabled = true;
            chart1.ChartAreas[0].AxisY.Minimum = GRAPH_Y_MIN[graph_select_index];
            chart1.ChartAreas[0].AxisY.Maximum = GRAPH_Y_MAX[graph_select_index];
            chart1.ChartAreas[0].AxisY.Interval = GRAPH_Y_INT[graph_select_index];
            chart1.ChartAreas[0].AxisY.LabelAutoFitMaxFontSize = 10;
            chart1.ChartAreas[0].AxisY.LabelStyle.ForeColor = Color.Black;
            chart1.ChartAreas[0].AxisY.MajorGrid.Enabled = true;
            chart1.ChartAreas[0].AxisY.MajorGrid.LineColor = Color.DarkGray;
            chart1.ChartAreas[0].AxisY.MinorGrid.Enabled = true;
            chart1.ChartAreas[0].AxisY.MinorGrid.LineColor = Color.LightGray;
            // ダミーデータ
            chart1.Series[0].Points.AddXY(10, 0);
            

            my_measure_val_display();

            FormUpdateTimer.Enabled = true;
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            try
            {
                if (serialPort1.IsOpen == true)
                {
                    serialPort1.Close();
                }
                my_logfile_output_stop();

                // 初期設定ファイル書き込み
                string file_path = Path.Combine(System.Environment.CurrentDirectory, Constants.INIT_FILE_NAME);
                my_Init_File_Save(file_path, false);

            }
            catch
            {
            }
        }

        private void FormUpdateTimer_Tick(object sender, EventArgs e)
        {
            try
            {
                byte[] tmp_serial_data = new byte[Constants.MEASUREMENT_TOTAL_DATA_SIZE];
                int tmp_read_idx = serial_rcv_data_read_idx;
                bool tmp_full = serial_rcv_data_full;
                int rcv_data_count = 0;
                string serial_debug_str = "";
                while (serial_rcv_data_read_idx != serial_rcv_data_write_idx || serial_rcv_data_full == true)
                {
                    if (serial_received_buff[tmp_read_idx] == Constants.MEASUREMENT_HEADER1)
                    {   // 先頭D
                        rcv_data_count = 0;
                        tmp_serial_data[rcv_data_count++] = serial_received_buff[tmp_read_idx++];
                        tmp_read_idx &= (SERIAL_RECEIVED_BUFF_SIZE - 1);
                        tmp_full = false;

                        while (tmp_read_idx != serial_rcv_data_write_idx || tmp_full == true)
                        {
                            tmp_serial_data[rcv_data_count++] = serial_received_buff[tmp_read_idx++];
                            tmp_read_idx &= (SERIAL_RECEIVED_BUFF_SIZE - 1);
                            tmp_full = false;

                            if (rcv_data_count >= tmp_serial_data.Length)
                            {
                                break;
                            }
                        }

                        if (rcv_data_count >= tmp_serial_data.Length)
                        {   // データ数OK
                            //データチェック M,...,\r,\n
                            if (tmp_serial_data[0] == Constants.MEASUREMENT_HEADER1 && tmp_serial_data[tmp_serial_data.Length - 2] == Constants.MEASUREMENT_FOOTER1 && tmp_serial_data[tmp_serial_data.Length - 1] == Constants.MEASUREMENT_FOOTER2)
                            {
                                byte[] tmp_byte = new byte[10];
                                string tmp_text = "";
                                double tmp_dbl = 0;

                                // 電圧
                                int data_top_idx = Constants.MEASUREMENT_VOLTAGE_TOP_IDX;
                                for (int fi = 0; fi < Constants.MEASUREMENT_VOLTAGE_DATA_SIZE; fi++)
                                {
                                    tmp_byte[fi] = tmp_serial_data[data_top_idx + fi];
                                    tmp_byte[fi+1] = 0x00;
                                }
                                tmp_text = System.Text.Encoding.ASCII.GetString(tmp_byte);
                                if (double.TryParse(tmp_text, out tmp_dbl) == true)
                                {
                                    myMeasureVal.voltage = tmp_dbl;
                                }
                                // 電流
                                data_top_idx = Constants.MEASUREMENT_CURRENT_TOP_IDX;
                                for (int fi = 0; fi < Constants.MEASUREMENT_CURRENT_DATA_SIZE; fi++)
                                {
                                    tmp_byte[fi] = tmp_serial_data[data_top_idx + fi];
                                    tmp_byte[fi + 1] = 0x00;
                                }
                                tmp_text = System.Text.Encoding.ASCII.GetString(tmp_byte);
                                if (double.TryParse(tmp_text, out tmp_dbl) == true)
                                {
                                    myMeasureVal.current = tmp_dbl;
                                }
                                // 容量
                                data_top_idx = Constants.MEASUREMENT_CAPACITANCE_TOP_IDX;
                                for (int fi = 0; fi < Constants.MEASUREMENT_CAPACITANCE_DATA_SIZE; fi++)
                                {
                                    tmp_byte[fi] = tmp_serial_data[data_top_idx + fi];
                                    tmp_byte[fi + 1] = 0x00;
                                }
                                tmp_text = System.Text.Encoding.ASCII.GetString(tmp_byte);
                                if (double.TryParse(tmp_text, out tmp_dbl) == true)
                                {
                                    myMeasureVal.capacitance = tmp_dbl;
                                }
                                // 高抵抗値
                                data_top_idx = Constants.MEASUREMENT_RESISTANCE_H_TOP_IDX;
                                for (int fi = 0; fi < Constants.MEASUREMENT_RESISTANCE_H_DATA_SIZE; fi++)
                                {
                                    tmp_byte[fi] = tmp_serial_data[data_top_idx + fi];
                                    tmp_byte[fi + 1] = 0x00;
                                }
                                tmp_text = System.Text.Encoding.ASCII.GetString(tmp_byte);
                                if (double.TryParse(tmp_text, out tmp_dbl) == true)
                                {
                                    myMeasureVal.resistance_h = tmp_dbl;
                                }
                                // 低抵抗値
                                data_top_idx = Constants.MEASUREMENT_RESISTANCE_L_TOP_IDX;
                                for (int fi = 0; fi < Constants.MEASUREMENT_RESISTANCE_L_DATA_SIZE; fi++)
                                {
                                    tmp_byte[fi] = tmp_serial_data[data_top_idx + fi];
                                    tmp_byte[fi + 1] = 0x00;
                                }
                                tmp_text = System.Text.Encoding.ASCII.GetString(tmp_byte);
                                if (double.TryParse(tmp_text, out tmp_dbl) == true)
                                {
                                    myMeasureVal.resistance_l = tmp_dbl;
                                }
                                // 周波数
                                data_top_idx = Constants.MEASUREMENT_FREQUENCY_TOP_IDX;
                                for (int fi = 0; fi < Constants.MEASUREMENT_FREQUENCY_DATA_SIZE; fi++)
                                {
                                    tmp_byte[fi] = tmp_serial_data[data_top_idx + fi];
                                    tmp_byte[fi + 1] = 0x00;
                                }
                                tmp_text = System.Text.Encoding.ASCII.GetString(tmp_byte);
                                if (double.TryParse(tmp_text, out tmp_dbl) == true)
                                {
                                    myMeasureVal.frequency = tmp_dbl;
                                }

                                //int tmp_level = 0;
                                //tmp_level = tmp_serial_data[3];
                                //tmp_level = (tmp_level << 8) | (tmp_serial_data[2] & 0x7F);
                                //if ((tmp_serial_data[2] & 0x80) != 0)
                                //{
                                //    tmp_level += 128;
                                //}
                                //// デシベルに変換
                                ////double db_tmp_level = tmp_level;
                                //db_tmp_level = tmp_level;
                                //db_tmp_level = (0.04 * db_tmp_level) - 81.2;
                                //string tmp_level_str = string.Format("{0}", db_tmp_level);

                                //string out_str = "";

                                //db_tmp_level_val = (int)db_tmp_level;
                                //textbx_input_db.Text = db_tmp_level_val.ToString();



                                //if (freq_auto_set_data_rcv_flag == true)
                                //{   // スリーブ中
                                //    out_str = "\"" + freq_auto_set_freq_val.ToString() + "\",\"" + tmp_level_str + "\"";


                                //    //グラフのデータを追加
                                //    chart1.Series[0].Points.AddXY((double)freq_auto_set_freq_val, db_tmp_level);
                                //    //chart1.Series[0].Points.AddXY(freq_auto_set_freq_val.ToString(), textbx_input_db.Text);
                                //}
                                //else
                                //{   // 周波数手動設定
                                //    out_str = "\"" + freq_manual_set_freq_val.ToString() + "\",\"" + tmp_level_str + "\"";
                                //}

                                //if (logfile_out_flag == true && sw_serial_log != null)
                                //{   // ログファイルへ出力
                                //    sw_serial_log.WriteLine(out_str);
                                //}

                                serial_debug_str += "\n";
                                if (chkbx_serial_RX_Ascii.Checked == true)
                                {
                                    serial_debug_str += Encoding.ASCII.GetString(tmp_serial_data, 0, rcv_data_count);
                                }
                                else
                                {
                                    for (int fi = 0; fi < rcv_data_count; fi++)
                                    {
                                        serial_debug_str += string.Format(" {0:X2}", tmp_serial_data[fi]);
                                    }
                                }

                                // 次のデータへ
                                serial_rcv_data_read_idx = tmp_read_idx;
                                serial_rcv_data_full = false;

                                if (measure_start_flag == true)
                                {
                                    my_measurement_request_command_send();
                                }
                            }
                            else
                            {   // データ不正 次のデータへ
                                if (chkbx_serial_RX_Ascii.Checked == true)
                                {
                                    serial_debug_str += Encoding.ASCII.GetString(serial_received_buff, serial_rcv_data_read_idx, 1);
                                }
                                else
                                {
                                    serial_debug_str += string.Format(" {0:X2}", serial_received_buff[serial_rcv_data_read_idx]);
                                }

                                serial_rcv_data_read_idx++;
                                serial_rcv_data_read_idx &= (SERIAL_RECEIVED_BUFF_SIZE - 1);
                                serial_rcv_data_full = false;
                            }
                        }
                        else
                        {   // データ数が足りない
                            break;
                        }
                    }
                    else
                    {   // 先頭Dでない 次のデータへ
                        if (chkbx_serial_RX_Ascii.Checked == true)
                        {
                            serial_debug_str += Encoding.ASCII.GetString(serial_received_buff, serial_rcv_data_read_idx, 1);
                        }
                        else
                        {
                            serial_debug_str += string.Format(" {0:X2}", serial_received_buff[serial_rcv_data_read_idx]);
                        }

                        serial_rcv_data_read_idx++;
                        serial_rcv_data_read_idx &= (SERIAL_RECEIVED_BUFF_SIZE - 1);
                        serial_rcv_data_full = false;
                    }
                }
                if (serial_debug_str != "")
                {
                    rtxtbx_serial_RX.Focus();
                    rtxtbx_serial_RX.AppendText(serial_debug_str);
                }

                DateTime dt_now = DateTime.Now;
                // 測定中？
                if (measure_start_flag == true)
                {
                    if (dt_now >= dt_MeasureVal_next_update)
                    {
                        double tmp_update_mill = (double)(num_set_cycle.Value * 1000);
                        dt_MeasureVal_next_update = dt_MeasureVal_next_update.AddMilliseconds(tmp_update_mill);
                        my_measure_val_display();
                    }
                }
                else
                {
                    myMeasureVal.Init();
                }

                // ログ出力中？
                if (logfile_out_flag == true && sw_serial_log != null)
                {
                    if (log_output_data_counter < (int)num_log_out_max.Value)
                    {
                        if (dt_now >= dt_log_next_output)
                        {

                            string str_log_out = "";
                            log_output_data_counter++;
                            str_log_out = my_get_log_output_data();
                            if (str_log_out != "")
                            {
                                sw_serial_log.WriteLine(str_log_out);
                            }
                            else
                            {
                                log_output_data_counter--;
                            }

                            double tmp_update_sec = (double)(num_log_output_cycle.Value);
                            dt_log_next_output = dt_log_next_output.AddSeconds(tmp_update_sec);
                        }
                    }
                    else
                    {
                        dt_log_next_output = DateTime.Now;
                    }
                }
            }
            catch
            {
            }
        }

        private void serialPort1_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            try
            {
                if (serialPort1.BytesToRead > 0)
                {
                    byte[] tmp_read_buff = new byte[serialPort1.BytesToRead];
                    serialPort1.Read(tmp_read_buff, 0, serialPort1.BytesToRead);

                    for (int fi = 0; fi < tmp_read_buff.Length; fi++)
                    {
                        if (serial_rcv_data_full == false)
                        {
                            serial_received_buff[serial_rcv_data_write_idx++] = tmp_read_buff[fi];

                            serial_rcv_data_write_idx &= (SERIAL_RECEIVED_BUFF_SIZE - 1);
                            if (serial_rcv_data_read_idx == serial_rcv_data_write_idx)
                            {
                                serial_rcv_data_full = true;
                            }
                        }
                    }
                }
            }
            catch
            {
            }
        }

        private void my_measurement_request_command_send()
        {
            byte[] send_data = new byte[64];
            try
            {
                if (serialPort1.IsOpen == true)
                {
                    try
                    {
                        send_data[0] = 0x4D;//M
                        send_data[1] = 0x52;//R
                        send_data[2] = 0x45;//E
                        send_data[3] = 0x0D;//\r
                        send_data[4] = 0x0A;//\n

                        serialPort1.Write(send_data, 0, 5);
                    }
                    catch
                    {
                    }
                }
            }
            catch
            {
            }
        }

        private void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
        {
            try
            {
            }
            catch
            {
            }
        }


        private void btn_correction_save_Click(object sender, EventArgs e)
        {
            CorrectionVal myCorrectionValtmp = new CorrectionVal();
            try
            {
                saveFileDialog1.Title = "Save As...";
                saveFileDialog1.Filter = "XML file(*.xml)|*.xml|All file(*.*)|*.*";
                if(saveFileDialog1.ShowDialog() == DialogResult.OK)
                {
                    my_Correction_File_Save(saveFileDialog1.FileName, false);
                }
            }
            catch
            {
            }
        }

        private void btn_correction_read_Click(object sender, EventArgs e)
        {
            CorrectionVal myCorrectionValtmp = new CorrectionVal();
            try
            {
                openFileDialog1.Title = "Open...";
                openFileDialog1.Filter = "XML file(*.xml)|*.xml|All file(*.*)|*.*";
                if (openFileDialog1.ShowDialog() == DialogResult.OK)
                {
                    my_Correction_File_Read(openFileDialog1.FileName);
                    // 設定値表示
                    my_setting_val_display_set();
                }
            }
            catch
            {
            }
        }

        private int my_Correction_File_Save(string file_name, bool save_flag)
        {
            int ret = -1;
            try
            {
                System.Xml.Serialization.XmlSerializer serializer = null;
                System.IO.FileStream fs = null;

                try
                {
                    CorrectionVal myCorrectionValtmp = new CorrectionVal();

                    myCorrectionValtmp.voltage_a = mySettingVal.voltage_a;
                    myCorrectionValtmp.voltage_b = mySettingVal.voltage_b;
                    myCorrectionValtmp.current_a = mySettingVal.current_a;
                    myCorrectionValtmp.current_b = mySettingVal.current_b;
                    myCorrectionValtmp.resistance_h_a = mySettingVal.resistance_h_a;
                    myCorrectionValtmp.resistance_h_b = mySettingVal.resistance_h_b;
                    myCorrectionValtmp.resistance_l_a = mySettingVal.resistance_l_a;
                    myCorrectionValtmp.resistance_l_b = mySettingVal.resistance_l_b;
                    myCorrectionValtmp.capacitance_a = mySettingVal.capacitance_a;
                    myCorrectionValtmp.capacitance_b = mySettingVal.capacitance_b;
                    myCorrectionValtmp.frequency_a = mySettingVal.frequency_a;
                    myCorrectionValtmp.frequency_b = mySettingVal.frequency_b;

                    serializer = new System.Xml.Serialization.XmlSerializer(typeof(CorrectionVal));
                    fs = new System.IO.FileStream(file_name, System.IO.FileMode.Create);
                    serializer.Serialize(fs, myCorrectionValtmp);

                    ret = 0;
                }
                catch
                {
                }
                finally
                {
                    if (fs != null)
                    {
                        fs.Close();
                        fs.Dispose();
                    }
                }
            }
            catch
            {
            }
            return ret;
        }

        private int my_Correction_File_Read(string file_name)
        {
            int ret = -1;
            try
            {
                // ファイル有無チェック
                if (File.Exists(file_name) == true)
                {
                    System.Xml.Serialization.XmlSerializer serializer = null;
                    System.IO.FileStream fs = null;
                    try
                    {
                        serializer = new System.Xml.Serialization.XmlSerializer(typeof(CorrectionVal));
                        fs = new System.IO.FileStream(file_name, System.IO.FileMode.Open);
                        CorrectionVal myCorrectionValtmp = (CorrectionVal)serializer.Deserialize(fs);

                        //読み込み正常
                        mySettingVal.voltage_a = myCorrectionValtmp.voltage_a;
                        mySettingVal.voltage_b = myCorrectionValtmp.voltage_b;
                        mySettingVal.current_a = myCorrectionValtmp.current_a;
                        mySettingVal.current_b = myCorrectionValtmp.current_b;
                        mySettingVal.resistance_h_a = myCorrectionValtmp.resistance_h_a;
                        mySettingVal.resistance_h_b = myCorrectionValtmp.resistance_h_b;
                        mySettingVal.resistance_l_a = myCorrectionValtmp.resistance_l_a;
                        mySettingVal.resistance_l_b = myCorrectionValtmp.resistance_l_b;
                        mySettingVal.capacitance_a = myCorrectionValtmp.capacitance_a;
                        mySettingVal.capacitance_b = myCorrectionValtmp.capacitance_b;
                        mySettingVal.frequency_a = myCorrectionValtmp.frequency_a;
                        mySettingVal.frequency_b = myCorrectionValtmp.frequency_b;
                        
                        ret = 0;
                    }
                    catch
                    {
                    }
                    finally
                    {
                        if (fs != null)
                        {
                            fs.Close();
                            fs.Dispose();
                        }
                    }
                }
            }
            catch
            {
            }
            return ret;
        }


        private int my_Init_File_Save(string file_name, bool save_flag)
        {
            int ret = -1;
            try
            {
                System.Xml.Serialization.XmlSerializer serializer = null;
                System.IO.FileStream fs = null;

                try
                {
                    serializer = new System.Xml.Serialization.XmlSerializer(typeof(SettingVal));
                    fs = new System.IO.FileStream(file_name, System.IO.FileMode.Create);
                    serializer.Serialize(fs, mySettingVal);


                    ret = 0;
                }
                catch
                {
                }
                finally
                {
                    if (fs != null)
                    {
                        fs.Close();
                        fs.Dispose();
                    }
                }
            }
            catch
            {
            }
            return ret;
        }

        private int my_Init_File_Read(string file_name)
        {
            int ret = -1;
            try
            {
                // ファイル有無チェック
                if (File.Exists(file_name) == true)
                {
                    System.Xml.Serialization.XmlSerializer serializer = null;
                    System.IO.FileStream fs = null;
                    try
                    {
                        serializer = new System.Xml.Serialization.XmlSerializer(typeof(SettingVal));
                        fs = new System.IO.FileStream(file_name, System.IO.FileMode.Open);
                        SettingVal mySettingValtmp = (SettingVal)serializer.Deserialize(fs);

                        //読み込み正常
                        mySettingVal = mySettingValtmp;

                        ret = 0;
                    }
                    catch
                    {
                    }
                    finally
                    {
                        if (fs != null)
                        {
                            fs.Close();
                            fs.Dispose();
                        }
                    }
                }
            }
            catch
            {
            }
            return ret;
        }

        private void my_setting_val_display_set()
        {
            double tmp_dbl = 0;
            try
            {
                setting_val_input_disabled = true;

                // 設定値
                tmp_dbl = mySettingVal.measurement_cycle;
                num_set_cycle.Value = (decimal)tmp_dbl;
                num_log_output_cycle.Value = mySettingVal.log_output_cycle;
                num_log_out_max.Value = mySettingVal.log_output_max;

                // 補正値
                num_vol_a.Value = (decimal)mySettingVal.voltage_a;
                num_vol_b.Value = (decimal)mySettingVal.voltage_b;
                num_cur_a.Value = (decimal)mySettingVal.current_a;
                num_cur_b.Value = (decimal)mySettingVal.current_b;
                num_hres_a.Value = (decimal)mySettingVal.resistance_h_a;
                num_hres_b.Value = (decimal)mySettingVal.resistance_h_b;
                num_lres_a.Value = (decimal)mySettingVal.resistance_l_a;
                num_lres_b.Value = (decimal)mySettingVal.resistance_l_b;
                num_cap_a.Value = (decimal)mySettingVal.capacitance_a;
                num_cap_b.Value = (decimal)mySettingVal.capacitance_b;
                num_freq_a.Value = (decimal)mySettingVal.frequency_a;
                num_freq_b.Value = (decimal)mySettingVal.frequency_b;

                setting_val_input_disabled = false;
            }
            catch
            {
            }
        }
        private void my_setting_val_display_get()
        {
            try
            {
                if (setting_val_input_disabled == false)
                {   // 画面に設定値設定中以外
                    // 設定値
                    mySettingVal.measurement_cycle = (double)num_set_cycle.Value;
                    mySettingVal.log_output_cycle = (int)num_log_output_cycle.Value;
                    mySettingVal.log_output_max = (int)num_log_out_max.Value;

                    // 補正値
                    mySettingVal.voltage_a = (double)num_vol_a.Value;
                    mySettingVal.voltage_b = (double)num_vol_b.Value;
                    mySettingVal.current_a = (double)num_cur_a.Value;
                    mySettingVal.current_b = (double)num_cur_b.Value;
                    mySettingVal.resistance_h_a = (double)num_hres_a.Value;
                    mySettingVal.resistance_h_b = (double)num_hres_b.Value;
                    mySettingVal.resistance_l_a = (double)num_lres_a.Value;
                    mySettingVal.resistance_l_b = (double)num_lres_b.Value;
                    mySettingVal.capacitance_a = (double)num_cap_a.Value;
                    mySettingVal.capacitance_b = (double)num_cap_b.Value;
                    mySettingVal.frequency_a = (double)num_freq_a.Value;
                    mySettingVal.frequency_b = (double)num_freq_b.Value;
                }
            }
            catch
            {
            }
        }

        private void num_setting_val_ValueChanged(object sender, EventArgs e)
        {
            try
            {
                my_setting_val_display_get();
            }
            catch
            {
            }
        }

        private void my_measure_val_display()
        {
            double tmp_dbl = 0;
            try
            {

                // 電圧
                txtbx_vol_1.Text = string.Format("{0:0.000}", myMeasureVal.voltage);
                double tmp_vol = myMeasureVal.voltage * mySettingVal.voltage_a + mySettingVal.voltage_b;
                txtbx_vol_2.Text = string.Format("{0:0.000}", tmp_vol);
                // 電流
                txtbx_cur_1.Text = string.Format("{0:0.0000}", myMeasureVal.current);
                double tmp_cur = myMeasureVal.current * mySettingVal.current_a + mySettingVal.current_b;
                txtbx_cur_2.Text = string.Format("{0:0.0000}", tmp_cur);
                // 電力
                txtbx_pow_1.Text = string.Format("{0:0.000}", myMeasureVal.voltage * myMeasureVal.current);
                tmp_dbl = tmp_vol * tmp_cur;
                txtbx_pow_2.Text = string.Format("{0:0.000}", tmp_dbl);
                // 高抵抗値
                if (myMeasureVal.resistance_h >= Constants.MEASUREMENT_RESISTANCE_H_RANGE_OVER_VAL)
                {   // 1.0MΩ以上 レンジオーバー
                    txtbx_hres_1.Text = Constants.MEASUREMENT_RESISTANCE_H_RANGE_OVER_MSG;
                    txtbx_hres_2.Text = Constants.MEASUREMENT_RESISTANCE_H_RANGE_OVER_MSG;
                }
                else
                {
                    txtbx_hres_1.Text = string.Format("{0:0.0}", myMeasureVal.resistance_h);
                    tmp_dbl = myMeasureVal.resistance_h * mySettingVal.resistance_h_a + mySettingVal.resistance_h_b;
                    txtbx_hres_2.Text = string.Format("{0:0.0}", tmp_dbl);
                }
                // 低抵抗値
                if (myMeasureVal.resistance_l >= Constants.MEASUREMENT_RESISTANCE_L_RANGE_OVER_VAL)
                {   // 10kΩ以上
                    txtbx_lres_1.Text = Constants.MEASUREMENT_RESISTANCE_L_RANGE_OVER_MSG;
                    txtbx_lres_2.Text = Constants.MEASUREMENT_RESISTANCE_L_RANGE_OVER_MSG;
                }
                else
                {
                    txtbx_lres_1.Text = string.Format("{0}", myMeasureVal.resistance_l);
                    tmp_dbl = myMeasureVal.resistance_l * mySettingVal.resistance_l_a + mySettingVal.resistance_l_b;
                    txtbx_lres_2.Text = string.Format("{0}", tmp_dbl);
                }
                // 容量
                tmp_dbl = myMeasureVal.capacitance;
                if (tmp_dbl < 0)
                {   // 負になったら０に制限
                    tmp_dbl = 0;
                }
                txtbx_cap_1.Text = string.Format("{0}", tmp_dbl);
                tmp_dbl = tmp_dbl * mySettingVal.capacitance_a + mySettingVal.capacitance_b;
                txtbx_cap_2.Text = string.Format("{0}", tmp_dbl);
                // 周波数
                txtbx_freq_1.Text = string.Format("{0}", myMeasureVal.frequency);
                tmp_dbl = myMeasureVal.frequency * mySettingVal.frequency_a + mySettingVal.frequency_b;
                txtbx_freq_2.Text = string.Format("{0}", tmp_dbl);
            }
            catch
            {
            }
        }

        private void btn_serial_open_Click(object sender, EventArgs e)
        {
            bool error_flag = false;
            byte[] send_data = new byte[64];
            try
            {
                if (serialPort1.IsOpen == false)
                {
                    serialPort1.PortName = cmbbx_port.SelectedItem.ToString();
                    if (0 <= cmbbx_baudrate.SelectedIndex && cmbbx_baudrate.SelectedIndex < baud_rate_select_datas.Length)
                    {
                        serialPort1.BaudRate = baud_rate_select_datas[cmbbx_baudrate.SelectedIndex];
                    }
                    else
                    {
                        error_flag = true;
                    }


                    if (error_flag == false)
                    {
                        try
                        {
                            serialPort1.WriteTimeout = 500;
                            serialPort1.ReadTimeout = 500;

                            serialPort1.Open();
                            if (serialPort1.IsOpen == true)
                            {
                                //backgroundWorker1.RunWorkerAsync();

                                //send_data[0] = 0x53;//S
                                //send_data[1] = 0x54;//T
                                //send_data[2] = 0x45;//E

                                try
                                {
                                    //serialPort1.Write(send_data, 0, send_data.Length);

                                    btn_serial_open.Enabled = false;
                                    btn_serial_close.Enabled = true;

                                    btn_measure_start.Enabled = true;
                                    btn_measure_stop.Enabled = false;
                                    btn_log_output_start.Enabled = true;
                                    btn_log_output_stop.Enabled = false;
                                }
                                catch
                                {
                                    serialPort1.Close();
                                    MessageBox.Show("シリアル通信エラー", "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                                }
                            }
                            else
                            {
                                MessageBox.Show("シリアルポートオープンエラー", "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                            }
                        }
                        catch
                        {
                            MessageBox.Show("シリアルポートオープンエラー", "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }
                    }
                    else
                    {
                        MessageBox.Show("シリアルポート　設定値エラー", "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
            }
            catch
            {
            }
        }

        private void btn_serial_close_Click(object sender, EventArgs e)
        {
            if (serialPort1.IsOpen == true)
            {
                serialPort1.Close();

                btn_serial_open.Enabled = true;
                btn_serial_close.Enabled = false;
                btn_measure_start.Enabled = false;
                btn_measure_stop.Enabled = false;

                my_logfile_output_stop();
                btn_log_output_start.Enabled = false;
                btn_log_output_stop.Enabled = false;
            }
        }

        private void btn_measure_start_Click(object sender, EventArgs e)
        {
            byte[] send_data = new byte[64];
            try
            {
                if (serialPort1.IsOpen == true)
                {
                    try
                    {
                        my_measurement_request_command_send();

                        btn_measure_start.Enabled = false;
                        btn_measure_stop.Enabled = true;
                        cmbbx_graph_item.Enabled = false;

                        // Graph設定
                        graph_select_index = cmbbx_graph_item.SelectedIndex;
                        for (int fi = 0; fi < graph_data.Length; fi++)
                        {
                            graph_data[fi] = 0;
                        }
                        graph_data_read_start_pos = 0;
                        graph_data_next_write_pos = 0;
                        my_graph_update(GRAPH_Y_UNIT[graph_select_index], GRAPH_Y_MIN[graph_select_index], GRAPH_Y_MAX[graph_select_index], GRAPH_Y_INT[graph_select_index]);
                        // Graph update timer
                        GraphUpdateTimer.Interval = (int)(num_set_cycle.Value * 1000);
                        graph_update_flag = true;
                        GraphUpdateTimer.Enabled = true;

                        dt_MeasureVal_next_update = DateTime.Now;
                        measure_start_flag = true;
                    }
                    catch
                    {
                        btn_measure_start.Enabled = true;
                        btn_measure_stop.Enabled = false;
                        cmbbx_graph_item.Enabled = true;
                    }
                }
            }
            catch
            {
            }
        }

        private void btn_measure_stop_Click(object sender, EventArgs e)
        {
            try
            {
                graph_update_flag = false;
                GraphUpdateTimer.Enabled = false;

                measure_start_flag = false;

                btn_measure_start.Enabled = true;
                btn_measure_stop.Enabled = false;
                cmbbx_graph_item.Enabled = true;
            }
            catch
            {
            }
        }

        private void btn_log_output_start_Click(object sender, EventArgs e)
        {
            try
            {
                my_logfile_output_start();
            }
            catch
            {
            }
        }

        private void btn_log_output_stop_Click(object sender, EventArgs e)
        {
            try
            {
                btn_log_output_start.Enabled = true;
                btn_log_output_stop.Enabled = false;

                my_logfile_output_stop();
            }
            catch
            {
            }
        }

        private void my_logfile_output_start()
        {
            int fi;
            string outStr = "";
            try
            {
                string log_file_path = "";
                string file_datetime = "";

                file_datetime = string.Format("{0:0000}{1:00}{2:00}_{3:00}{4:00}{5:00}", DateTime.Now.Year, DateTime.Now.Month, DateTime.Now.Day, DateTime.Now.Hour, DateTime.Now.Minute, DateTime.Now.Second);
                log_file_path = System.Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments) + LOG_FILE_NAME + file_datetime + LOG_FILE_EXTEN;

                try
                {
                    sw_serial_log = new StreamWriter(log_file_path);

                    // ヘッダ
                    for (fi = 0; fi < LOG_FILE_HEADER.Length; fi++)
                    {
                        if (fi >= 1)
                        {
                            outStr += ",";
                        }
                        outStr += "\"" + LOG_FILE_HEADER[fi] + "\"";
                    }
                    sw_serial_log.WriteLine(outStr);

                    btn_log_output_start.Enabled = false;
                    btn_log_output_stop.Enabled = true;

                    dt_log_next_output = DateTime.Now;
                    log_output_data_counter = 0;

                    logfile_out_flag = true;
                }
                catch
                {
                    if (sw_serial_log != null)
                    {
                        sw_serial_log.Close();
                        sw_serial_log.Dispose();
                        sw_serial_log = null;
                    }
                    btn_log_output_start.Enabled = true;
                    btn_log_output_stop.Enabled = false;
                }

            }
            catch
            {
            }
        }
        private void my_logfile_output_stop()
        {
            try
            {
                if (sw_serial_log != null)
                {
                    sw_serial_log.Close();
                    sw_serial_log.Dispose();
                    sw_serial_log = null;
                }

                logfile_out_flag = false;
            }
            catch
            {
            }
        }

        private void my_graph_update(string p_y_title, double p_axis_min, double p_axis_max, double p_axis_int)
        {
            try
            {
                chart1.Series[0].Points.Clear();

                chart1.ChartAreas[0].AxisY.Title = p_y_title;
                chart1.ChartAreas[0].AxisY.Minimum = p_axis_min;
                chart1.ChartAreas[0].AxisY.Maximum = p_axis_max;
                chart1.ChartAreas[0].AxisY.Interval = p_axis_int;

                int tmp_idx = graph_data_read_start_pos;
                for (int fi = 0; fi < graph_data_num; fi++)
                {
                    chart1.Series[0].Points.Add(new DataPoint(0, graph_data[tmp_idx]));

                    tmp_idx++;
                    if (tmp_idx >= GRAPH_DATA_BUFF_SIZE)
                    {
                        tmp_idx = 0;
                    }
                }
            }
            catch
            {
            }
        }

        private void GraphUpdateTimer_Tick(object sender, EventArgs e)
        {
            double set_data = 0;
            try
            {
                if (graph_update_flag == true)
                {   // グラフ更新中

                    // データセット
                    switch (graph_select_index)
                    {
                        case GRAPH_ITEM_VOLTAGE_INDEX:
                            set_data = myMeasureVal.voltage;
                            break;
                        case GRAPH_ITEM_CURRENT_INDEX:
                            set_data = myMeasureVal.current;
                            break;
                        case GRAPH_ITEM_POWER_INDEX:
                            set_data = myMeasureVal.voltage * myMeasureVal.current;
                            break;
                        case GRAPH_ITEM_RESISTANCE_H_INDEX:
                            set_data = myMeasureVal.resistance_h;
                            if (set_data >= Constants.MEASUREMENT_RESISTANCE_H_RANGE_OVER_VAL)
                            {
                                set_data = Constants.MEASUREMENT_RESISTANCE_H_RANGE_OVER_VAL;
                            }
                            break;
                        case GRAPH_ITEM_RESISTANCE_L_INDEX:
                            set_data = myMeasureVal.resistance_l;
                            if (set_data >= Constants.MEASUREMENT_RESISTANCE_L_RANGE_OVER_VAL)
                            {
                                set_data = Constants.MEASUREMENT_RESISTANCE_L_RANGE_OVER_VAL;
                            }
                            break;
                        case GRAPH_ITEM_CAPACITANCE_INDEX:
                            set_data = myMeasureVal.capacitance;
                            if (set_data < 0)
                            {   // 負になったら０に制限
                                set_data = 0;
                            }
                            break;
                        case GRAPH_ITEM_FREQUENCY_INDEX:
                            set_data = myMeasureVal.frequency;
                            break;
                    }

                    graph_data[graph_data_next_write_pos] = set_data;
                    // Div設定
                    //while (graph_data[graph_data_next_write_pos] > graph_div_val[graph_div_select_no])
                    //{
                    //    graph_div_select_no++;
                    //    if (graph_div_select_no >= graph_div_val.Length)
                    //    {
                    //        graph_div_select_no = graph_div_val.Length - 1;
                    //        break;
                    //    }
                    //}


                    //データ保存位置と読み込み先頭位置チェック
                    if (graph_data_next_write_pos == graph_data_read_start_pos)
                    {   //読み込み先頭位置をずらす
                        graph_data_read_start_pos++;
                        if (graph_data_read_start_pos >= GRAPH_DATA_BUFF_SIZE)
                        {
                            graph_data_read_start_pos = 0;
                        }
                    }
                    //次データ保存位置をずらす
                    graph_data_next_write_pos++;
                    if (graph_data_next_write_pos >= GRAPH_DATA_BUFF_SIZE)
                    {
                        graph_data_next_write_pos = 0;
                    }
                    //データ数加算
                    graph_data_num++;
                    if (graph_data_num > GRAPH_DATA_BUFF_SIZE)
                    {
                        graph_data_num = GRAPH_DATA_BUFF_SIZE;
                    }


                    // データ表示
                    my_graph_update(GRAPH_Y_UNIT[graph_select_index], GRAPH_Y_MIN[graph_select_index], GRAPH_Y_MAX[graph_select_index], GRAPH_Y_INT[graph_select_index]);

                    //my_graph_sensitivity_update((int)now_threshold * (int)THRESHOLD_MULTIPLIER);
                }
                else
                {   //　グラフ更新停止
                }
            }
            catch
            {
            }
        }

        private string my_get_log_output_data()
        {
            string ret_val = "";
            double tmp_dbl = 0;
            try
            {
                // no
                ret_val = "\"" + log_output_data_counter.ToString() + "\"";
                // time stamp
                ret_val += ",\"" + dt_log_next_output.ToString("yyyy/MM/dd") + "\"";
                ret_val += ",\"" + dt_log_next_output.ToString("HH:mm:ss.fff") + "\"";
                //ret_val += ",\"" + dt_MeasureVal_next_update.ToString("yyyy/MM/dd HH:mm:ss.ffff") + "\"";

                // 電圧
                ret_val += ",\"" + string.Format("{0:0.000}", myMeasureVal.voltage) +"\"";
                // 電流
                ret_val += ",\"" + string.Format("{0:0.0000}", myMeasureVal.current) + "\"";
                // 電力
                ret_val += ",\"" + string.Format("{0:0.000}", myMeasureVal.voltage * myMeasureVal.current) + "\"";
                // 高抵抗
                tmp_dbl = myMeasureVal.resistance_h;
                if (tmp_dbl >= Constants.MEASUREMENT_RESISTANCE_H_RANGE_OVER_VAL)
                {
                    tmp_dbl = Constants.MEASUREMENT_RESISTANCE_H_RANGE_OVER_VAL;
                }
                ret_val += ",\"" + string.Format("{0:0.0}", tmp_dbl) + "\"";
                // 低抵抗
                tmp_dbl = myMeasureVal.resistance_l;
                if (tmp_dbl >= Constants.MEASUREMENT_RESISTANCE_L_RANGE_OVER_VAL)
                {
                    tmp_dbl = Constants.MEASUREMENT_RESISTANCE_L_RANGE_OVER_VAL;
                }
                ret_val += ",\"" + string.Format("{0}", tmp_dbl) + "\"";
                // 容量
                tmp_dbl = myMeasureVal.capacitance;
                if (tmp_dbl < 0)
                {   // 負になったら０に制限
                    tmp_dbl = 0;
                }
                ret_val += ",\"" + string.Format("{0}", tmp_dbl) + "\"";
                // 周波数
                ret_val += ",\"" + string.Format("{0}", myMeasureVal.frequency) + "\"";
            }
            catch
            {
            }
            return ret_val;
        }

        private void btn_serial_rcv_log_clear_Click(object sender, EventArgs e)
        {
            try
            {
                rtxtbx_serial_RX.Text = "";
            }
            catch
            {
            }
        }

    }
}
